import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:pinch_zoom/pinch_zoom.dart';
import 'package:url_launcher/url_launcher.dart';

class FileReader {
  static const MethodChannel _channel = MethodChannel('file_reader');

  static Future<String> openFile(String path, {
    String title = '',
    String token = '',
    String localSizePermission = 'Quyen mo may'
  }) async {
    Map<String, String> map = {
      "path": path,
      "title": title,
      "token": token,
      "localSizePermission" : localSizePermission
    };
    if (kIsWeb) {
      if(path.endsWith(".pdf")) {
        await launch(
          path,
          forceSafariVC: true,
          forceWebView: true,
          webOnlyWindowName: '_self',
        );
        // html.window.open(path,"_self");
      } else {
        await launch(
          "https://view.officeapps.live.com/op/embed.aspx?src=" + path,
          forceSafariVC: true,
          forceWebView: true,
          webOnlyWindowName: '_self',
        );
        // html.window.open(
        //     "https://view.officeapps.live.com/op/embed.aspx?src=" + path,
        //     "_self");
      }
      return "Success";
    } else {
      final String result = await _channel.invokeMethod('openFile', map);
      return result;
    }
  }

  static Widget openPhoto(String url){
    return PinchZoom(
      child: Image.network(url),
      resetDuration: const Duration(milliseconds: 100),
      maxScale: 2.5,
      onZoomStart: (){print('Start zooming');},
      onZoomEnd: (){print('Stop zooming');},
    );
  }
}
